%{
/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <stdio.h>
#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_CC_ALGO_PARSER_L_L
#include "osm_cc_algo_parser_y.h"
extern char *__cc_algo_parser_yylval;
extern char *trim_whitespace(char *str);
#define SAVE_POS save_pos()
static void save_pos();

extern int cc_algo_column_num;
extern int cc_algo_line_num;

%}


%option nounput noinput
STRING [a-zA-Z0-9'_''\-']*

%%

\#.*\n                                       { SAVE_POS; return COMMENT; }
[ \t]*\n                       	             { SAVE_POS; } /* ignore end of line and empty lines*/
[' ']*                                       { SAVE_POS; } /* ignore spaces */
[ \t]*algo_release_date:.*\n                 { SAVE_POS; } /* ignore info only */
[ \t]*algo_description:.*\n                  { SAVE_POS; } /* ignore info only */
[ \t]*algo_author:.*\n                       { SAVE_POS; } /* ignore info only */
[ \t]*supported_devices:.*\n                 { SAVE_POS; } /* ignore info only */
[ \t]*ppcc_counters_name_list:.*\n           { SAVE_POS; } /* ignnore, for ibdiagnet only */
,                                            { SAVE_POS; return COMMA; }
"."                                          { SAVE_POS; return DOT; }
"["                                          { SAVE_POS; return LEFT_BRACKET; }
"]"                                          { SAVE_POS; return RIGHT_BRACKET; }
"("                                          { SAVE_POS; return LEFT_PARENTHESIS; }
")"                                          { SAVE_POS; return RIGHT_PARENTHESIS; }
[ \t]*ca_algo_start[ \t]*      	             { SAVE_POS; return CA_ALGO_START; }
[ \t]*ca_algo_end[ \t]*        	             { SAVE_POS; return CA_ALGO_END; }
[ \t]*name:                    	             { SAVE_POS; return ALGO_NAME; }
[ \t]*version:                               { SAVE_POS; return ALGO_VERSION; }
[ \t]*ppcc_parameter_name_list:              { SAVE_POS; return PPCC_PARAMETER_NAME_LIST; }
[ \t]*ppcc_parameter_min_vals:               { SAVE_POS; return PPCC_PARAMETER_MIN_VALS; }
[ \t]*ppcc_parameter_max_vals:               { SAVE_POS; return PPCC_PARAMETER_MAX_VALS; }
0[xX][0-9a-fA-F]+ {
			free(__cc_algo_parser_yylval);
			__cc_algo_parser_yylval = strdup(yytext);
			SAVE_POS;
			return NUMBER;
		  }
[0-9]+	{
		free(__cc_algo_parser_yylval);
		__cc_algo_parser_yylval = strdup(yytext);
		SAVE_POS;
		return NUMBER;
        }
{STRING} {
		free(__cc_algo_parser_yylval);
		__cc_algo_parser_yylval = strdup(trim_whitespace(yytext));
		SAVE_POS;
		return WORD;
	}
%%


static void save_pos()
{
	int i;
	for (i = 0; i < yyleng; i++) {
		if (yytext[i] == '\n') {
			cc_algo_line_num++;
			cc_algo_column_num = 1;
		} else {
			cc_algo_column_num++;
		}
	}
}
